@@ -1,3 +1,10 @@
+class DeprecationError(Exception):
+    """
+    Class raised for use of functionality that is no longer supported by icepyx.
+    """
+    pass
+
+
 class QueryError(Exception):
     """
     Base class for Query object exceptions
@@ -20,3 +27,5 @@ class NsidcQueryError(QueryError):
 
     def __str__(self):
         return f"{self.msgtxt}: {self.errmsg}"
+
+    
